/**
 -------------------------------------------------------------------------------------------------------------------------------
This file provides boilerplate templates for interfacing with the GUI Composer framework.
For further information, see the available options under the Help main menu in the Designer.
-------------------------------------------------------------------------------------------------------------------------------
**/

import { GcUtils } from './components/@ti/gc-core-assets/lib/GcUtils';
import { GcConsole } from './components/@ti/gc-core-assets/lib/GcConsole';
import { bindingRegistry, streamingDataEventType, WidgetModel } from './components/@ti/gc-core-databind/lib/CoreDatabind';
import { GcWidget } from './components/@ti/gc-widget-base/lib/GcWidget';
import { ActionRegistry } from './components/@ti/gc-widget-menu/lib/ActionRegistry';
import { codecRegistry } from "./components/@ti/gc-target-configuration/lib/TargetConfiguration";

const console = new GcConsole('myapp'); // creates a console instance with name 'myapp'
GcConsole.setLevel('myapp', 4);         // enable console output for myapp console instance
console.info('index.js is loaded...');

const FileSaver = {};
const defaultFile = 'TMP110EVM_Data.csv';
const fileHeader = 'Temperature C, Temperature F, Time, Alert Polarity, Alert bit, T_High Limit, T_Low Limit\n';
const isOnline = typeof nw == 'undefined' || typeof process == 'undefined';

let dbgOnline = false;
if (isOnline | dbgOnline) {
    streamSaver.myurl = location.href.split('/');
    streamSaver.myurl.pop();
    streamSaver.myurl = streamSaver.myurl.join('/') + '/';
    streamSaver.mitm = `${streamSaver.myurl}mitm.html?version=${streamSaver.version.full}`;
    let mywriter;
    const myTextEnc = new TextEncoder();
    FileSaver.saveStart = function (filename) {
        if (!filename) filename = defaultFile;
        const mystream = streamSaver.createWriteStream(filename);
        FileSaver.saveAbort();
        mywriter = mystream.getWriter();
        FileSaver.saveData(fileHeader);
    };
    FileSaver.saveStop = function () {
        if (mywriter) {
            mywriter.close();
            mywriter = null;
        }
    };
    FileSaver.saveAbort = function () {
        if (mywriter) {
            mywriter.abort('reason');
            mywriter = null;
        }
    };
    FileSaver.saveData = function (data) {
        if (mywriter) {
            mywriter.write(myTextEnc.encode(data));
        }
    };
    window.addEventListener('unload', (event) => {
        FileSaver.saveAbort();
    });
} else {
    // node fs API: https://nodejs.org/api/fs.html
    const fs = require('fs');
    FileSaver.saveLog = false;
    FileSaver.saveFile;
    let saveStarted = undefined;
    // https://nwjs.readthedocs.io/en/nw13/References/File%20Dialogs/#attribute-nwsaveas
    document.querySelector('#file_input').addEventListener('change', e => {
        let f = document.querySelector('#file_input');
        FileSaver.saveFile = f.value;
        f.value = "";
        FileSaver.saveLog = true;
    });

    FileSaver.saveStart = function (filename) {
        let f = document.querySelector('#file_input');
        saveStarted = true;
        f.setAttribute('nwsaveas', defaultFile);
        f.click();
    };
    FileSaver.saveStop = function () {
        FileSaver.saveLog = false;
    };
    FileSaver.saveAbort = function () {
        FileSaver.saveLog = false;
    };
    FileSaver.saveData = function (data) {
        if (!FileSaver.saveLog) return;
        if (saveStarted) {
            saveStarted = false;
            fs.writeFileSync(FileSaver.saveFile, fileHeader, { flag: 'w' /* overwrite */ });
        }
        fs.writeFileSync(FileSaver.saveFile, data, { flag: 'a' /* append to file */ });
    };
}

/**
 -------------------------------------------------------------------------------------------------------------------------------
Boilerplate code for databinding

Add custom computed value databindings here, using the following method:

syntax: bindingRegistry.bind(targetBinding, sourceBinding, [getter], [setter]);
    param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
    param sourceBinding - single binding string or expression, or array of binding strings for multi-way binding.
    param getter - (optional) - custom getter function for computing the targetBinding value(s) based on sourceBinding value(s).
    param setter - (optional) - custom setter function for computing the sourceBinding value(s) based on targetBinding value(s).

  (async () => {
      await bindingRegistry.waitForModelReady('widget');        // widget model, a built-in model
      await bindingRegistry.waitForModelReady('targetModelId'); // target model, gc-model-program, gc-model-streaming, etc...
      bindingRegistry.bind('widget.inputWidgetId.value', 'model.targetModelId.targetVariable');
      bindingRegistry.bind('widget.labelWidgetId.label', 'widget.inputWidgetId.value');
   })();
-------------------------------------------------------------------------------------------------------------------------------
**/

// Example code:
//
(async () => {
    /* Wait for widget and target models to be ready */
    await bindingRegistry.waitForModelReady('widget');
    await bindingRegistry.waitForModelReady('reg');
    
    bindingRegistry.bind("Temp.label", "Q(reg.Temp_Result.Temp_Result, reg.Configuration.Extended_Mode ? 0 : 1)", x => x + "°C");
    bindingRegistry.bind("Temp_F.label", "Q(reg.Temp_Result.Temp_Result, reg.Configuration.Extended_Mode ? 0 : 1)", x => ((x * 9/5) + 32) + "°F");
    
    /*bindingRegistry.bind("line1.point", "Q(reg.Temp_Result.Temp_Result, reg.Configuration.Extended_Mode ? 0 : 1)", y => [y]);*/
     bindingRegistry.bind("line1.point", "Q(reg.Temp_Result.Temp_Result, reg.Configuration.Extended_Mode ? 0 : 1)", function(value) {
        let d = new Date();
        let timestmp = d.toLocaleString().replace(',','');
        let pol = bindingRegistry.getBinding('reg.Configuration.Polarity').getValue();
	    let bit = bindingRegistry.getBinding('reg.Configuration.Alert_Bit').getValue();
	    let alert_thigh = bindingRegistry.getBinding("Q(reg.THigh_Limit.THigh_Limit, reg.Configuration.Extended_Mode ? 0 : 1)").getValue();
	    let alert_tlow = bindingRegistry.getBinding("Q(reg.TLow_Limit.TLow_Limit, reg.Configuration.Extended_Mode ? 0 : 1)").getValue();
	    let temp_f = ((value * 9/5) + 32);
        FileSaver.saveData(value + ',' + temp_f + ',' + timestmp + ',' + pol + 'b' + ',' + bit + 'b' + ',' + alert_thigh + ',' + alert_tlow + ',' + '\n');
        return [value];
     });
    bindingRegistry.bind("line2.point", "Q(reg.TLow_Limit.TLow_Limit, reg.Configuration.Extended_Mode ? 0 : 1)", y => [y]);
    bindingRegistry.bind("line3.point", "Q(reg.THigh_Limit.THigh_Limit, reg.Configuration.Extended_Mode ? 0 : 1)", y => [y]);
    
    bindingRegistry.bind("input_thigh.value", "Q(reg.THigh_Limit.THigh_Limit, reg.Configuration.Extended_Mode ? 0 : 1)",
            function(value) { 
				return value;
			},
			function(value) {
				return value;
			} );
	bindingRegistry.bind("THigh_Limit.label", "Q(reg.THigh_Limit.THigh_Limit, reg.Configuration.Extended_Mode ? 0 : 1)", x => "0x" + x.toString(16).toUpperCase() );
    
    bindingRegistry.bind("input_tlow.value", "Q(reg.TLow_Limit.TLow_Limit, reg.Configuration.Extended_Mode ? 0 : 1)",
            function(value) { 
				return value;
			},
			function(value) {
				return value;
			} );
	bindingRegistry.bind("TLow_Limit.label", "Q(reg.TLow_Limit.TLow_Limit, reg.Configuration.Extended_Mode ? 0 : 1)", x => "0x" + x.toString(16).toUpperCase() );
			
    bindingRegistry.bind("alert_indicator.on", "reg.Configuration.Alert_Bit", 
	    function(value) {
	        let pol = bindingRegistry.getBinding('reg.Configuration.Polarity').getValue();
	        let bit = bindingRegistry.getBinding('reg.Configuration.Alert_Bit').getValue();
	        if(pol === 0 && bit === 0){
	            value = true;
	            return value;
	        }
	        else if(pol == 1 && bit == 1){
	            value = true;
	            return value;
	        }
	        else{
	            value = false;
	            return value;
	        }
	   });
    
    /*bindingRegistry.getBinding('reg.Temp_Result.Temp_Result').addEventListener(streamingDataEventType, ({data}) => {
        let d = new Date();
        let timestmp = d.toLocaleString().replace(',','');
        let pol = bindingRegistry.getBinding('reg.Configuration.Polarity').getValue();
	    let bit = bindingRegistry.getBinding('reg.Configuration.Alert_Bit').getValue();
        FileSaver.saveData(data + ',' + timestmp + ',' + pol + 'b' + ',' + bit + 'b' + '\n');
    });*/
    
    bindingRegistry.bind('select_conversionrate.selectedIndex', 'reg.Configuration.Conversion_Rate');
    bindingRegistry.bind('select_pollingrate.selectedValue', 'register_map.autoReadValue', x => {
        return x;
    }, y => {
        return y;
    });
//
//     /* A simple computed values based on simple expression */
//     bindingRegistry.bind('widget.id.propertyName', "targetModelId.targetVariable == 1 ? 'binding is one' : 'binding is not one'");
//
//     /* A custom two-way binding with custom getter and setter functions */
//     /* (setter is optional and getter only indicates one-way binding)   */
//     bindingRegistry.bind('widget.id.propertyName', 'targetModelId.targetVariable',
//         value => { return value*5/9 + 32; }, /* getter */
//         value => { return (value-32)*9/5; }  /* setter */
//     );
//
//     /* 1 to n bindings */
//     bindingRegistry.bind('widget.date.value', {
//         /* Dependant bindings needed in order to compute the date, in name/value pairs */
//             weekday: 'widget.dayOfWeek.selectedLabel',
//             day: 'widget.dayOfMonth.value',
//             month: 'widget.month.selectedLabel',
//             year: 'widget.year.value'
//         },
//         /* Getter for date computation */
//         function(values) {
//             /* compute and return the string value to bind to the widget with id 'date' */
//             return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
//         }
//     );
})();


/**
 -------------------------------------------------------------------------------------------------------------------------------
Boilerplate code for working with webcomponents in the application
-------------------------------------------------------------------------------------------------------------------------------
**/

const init = () => {
    // Add menubar product-name-clicked event listener
    GcWidget.querySelector('gc-widget-menubar').then(menubar => {
        menubar.addEventListener('product-name-clicked', () => window.open('https://dev.ti.com/gc', 'gc'));
    });
    GcWidget.querySelector('#button_data_start').then(b => b.addEventListener('click', () => {
        FileSaver.saveStart();
        button_data_start.disabled = true;
        button_data_stop.disabled = false;
    }));
    GcWidget.querySelector('#button_data_stop').then(b => b.addEventListener('click', () => {
        FileSaver.saveStop();
        button_data_start.disabled = false;
        button_data_stop.disabled = true;
        }));
    GcWidget.querySelector('#gen_call_reset').then(b => b.addEventListener('click', () => {
        /*var u2a = document.querySelector('ti-model-register').getCodec();*/
        const u2a = codecRegistry.getInstance('usb2any');
		var Cmd_I2C_Write = 2;
		var commandSent = u2a.sendCommandPacket(Cmd_I2C_Write, [0, 0, 1, 6]);
    }));
};
document.readyState === 'complete' ? init() : document.addEventListener('DOMContentLoaded', init);


/**
 -------------------------------------------------------------------------------------------------------------------------------
Boilerplate code for registering menu and toolbar action callback
-------------------------------------------------------------------------------------------------------------------------------
**/

ActionRegistry.registerAction('cmd_exit', {
    run() { GcUtils.isNW ? require('nw.gui').Window.get().close() : window.close(); }
});